package jalview.bin;

import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

/**
 * Collection of all known applet tags from JalviewLite
 * 
 * @author hansonr
 *
 */
@SuppressWarnings("serial")
public class AppletParams extends HashMap<String, String>
{

  private final static String[] params = { "alignpdbfiles",
      "ANNOTATIONCOLOUR_MAX", "ANNOTATIONCOLOUR_MIN", "annotations",
      "APPLICATION_URL", "automaticScrolling", "centrecolumnlabels",
      "debug", "defaultColour", "defaultColourNuc", "defaultColourProt",
      "embedded", "enableSplitFrame", "externalstructureviewer", "features",
      "file", "file2", "format", "heightScale", "hidefeaturegroups",
      "jalviewhelpurl", "jnetfile", "jpredfile", "label", "linkLabel_",
      "linkLabel_1", "linkURL_", "nojmol", "normaliseLogo",
      "normaliseSequenceLogo", "oninit", "PDBFILE", "PDBSEQ",
      "relaxedidmatch", "resolvetocodebase", "RGB", "scaleProteinAsCdna",
      "scoreFile", "separator", "sequence", "showAnnotation", "showbutton",
      "showConsensus", "showConsensusHistogram", "showConservation",
      "showfeaturegroups", "showFeatureSettings", "showFullId",
      "showGroupConsensus", "showGroupConservation", "showOccupancy",
      "showQuality", "showSequenceLogo", "showTreeBootstraps",
      "showTreeDistances", "showUnconserved", "showUnlinkedTreeNodes",
      "sortBy", "sortByTree", "tree", "treeFile", "upperCase",
      "userDefinedColour", "widthScale", "windowHeight", "windowWidth",
      "wrap", };

  public AppletParams(Map<String, String> info)
  {
    for (int i = params.length; --i >= 0;)
    {
      put(params[i], info.get(params[i]));
    }
  }

  public String getParam(String param, String def)
  {
    String val = get(param);
    return (val != null ? val : def);
  }

  // <applet
  // code="jalview.bin.JalviewLite" width="140" height="35"
  // archive="jalviewApplet.jar,JmolApplet-14.6.4_2016.10.26.jar,java-json.jar,json_simple-1.1.jar">
  // <param name="permissions" value="sandbox"/>
  // <param name="file" value="uniref50.fa"/>
  // <param name="treeFile" value="ferredoxin.nw"/>
  // <param name="userDefinedColour" value="C=yellow; R,K,H=FF5555;
  // D,E=5555FF"/>
  // <param name="sortByTree" value="True"/>
  // <param name="showSequenceLogo" value="true"/>
  // <param name="showGroupConsensus" value="true"/>
  // <param name="showFullId" value="false"/>
  // <param name="linkLabel_1" value="Uniprot"/>
  // <param name="linkUrl_1"
  // value="http://www.uniprot.org/uniprot/$SEQUENCE_ID$"/>
  // <param name="linkLabel_2" value="EMBL-EBI Search"/>
  // <param name="linkUrl_2"
  // value="http://www.ebi.ac.uk/ebisearch/search.ebi?db=allebi&query=$SEQUENCE_ID$"/>
  // <param name="APPLICATION_URL"
  // value="http://www.jalview.org/services/launchApp"/>
  // </applet>
  //
  public AppletParams(String outerHTML)
  {
    String[] tokens = outerHTML.split("<param");
    outerHTML = tokens[0];
    String code = getAttr(outerHTML, "code");
    if (!code.equals("jalview.bin.JalviewLite"))
    {
      return;
    }
    for (int i = tokens.length; --i > 0;)
    {
      String param = tokens[i];
      String key = getAttr(param, "name");
      if (key != null)
      {
        String value = getAttr(param, "value");
        System.out.println("AppletParams " + key + " = \"" + value + "\"");
        put(key, value);
      }
    }
    put("_width", getAttr(outerHTML, "width"));
    put("_height", getAttr(outerHTML, "height"));
    put("_id", getAttr(outerHTML, "id"));
    put("_name", getAttr(outerHTML, "name"));
    put("_archive", getAttr(outerHTML, "archive"));
    put("_code", code);
  }

  public AppletParams()
  {
    // TODO Auto-generated constructor stub
  }

  public static AppletParams getAppletParams(String[] args,
          Vector<String> vargs)
  {
    AppletParams appletParams = new AppletParams();
    String resourcePath = null;
    for (int i = args.length; --i > 0;) // > 0 is correct, not >=0
    {
      if (args[i].startsWith("name=\"Info.resourcePath\""))
      {
        resourcePath = getAttr(args[i], "value");
        if (resourcePath.length() > 0 && !resourcePath.endsWith("/"))
        {
          resourcePath += "/";
        }
        break;
      }
    }
    for (int i = 1; i < args.length; i++)
    {
      String arg = args[i].trim();
      if (arg.startsWith("name="))
      {
        String appName = getAttr(arg, "name");
        String value = getAttr(arg, "value");
        String appletName = appName.toLowerCase();

        // note that Application arguments ARE case-sensitive, but
        // Applet.getParameter() is not.

        switch (appletName)
        {
        // tested, working:
        case "jnetfile":
          value = resourcePath + value;
          appName = null;
          break;
        case "jpredfile":
          value = resourcePath + value;
          appName = null;
          break;
        case "features":
          value = resourcePath + value;
          appName = null;
          break;
        case "pdbfile":
          appName = "PDBFILE";
          value = resourcePath + value;
          appName = null;
          break;
        case "tree":
        case "treefile":
          appName = "tree";
          value = resourcePath + value;
          appletName = null; // taken care of by Jalview
          break;
        // implemented; not tested:
        case "annotations":
          value = resourcePath + value;
          appName = null;
          break;
        case "defaultcolour":
          appName = "DEFAULT_COLOUR"; // defaultColour
          break;
        case "defaultcolournuc":
          appName = "DEFAULT_COLOUR_NUC";// defaultColourNuc
          break;
        case "defaultcolourprot":
          appName = "DEFAULT_COLOUR_PROT";// defaultColourProt";
          break;
        case "annotationcolour_max":
          appName = "ANNOTATIONCOLOUR_MAX";
          break;
        case "annotationcolour_min":
          appName = "ANNOTATIONCOLOUR_MIN";
          break;
        case "centrecolumnlabels":
          appName = null;
          break;
        case "enablesplitframe":
          appName = "enableSplitFrame";
          break;
        case "externalstructureviewer":
          break;
        case "alignpdbfiles":
          break;
        case "application_url":
          appName = "APPLICATION_URL";
          break;
        case "automaticscrolling":
          appName = "automaticScrolling";
          break;
        case "heightscale":
          appName = "heightScale";
          break;
        case "hidefeaturegroups":
          break;
        case "jalviewhelpurl":
          break;
        case "label":
          break;
        case "linklabel_":
          appName = "linkLabel_";
          break;
        case "linklabel_1":
          appName = "linkLabel_1";
          break;
        case "linkurl_":
          appName = "linkURL_";
          break;
        case "nojmol":
          break;
        case "normaliselogo":
          appName = "normaliseLogo";
          break;
        case "normalisesequencelogo":
          appName = "normaliseSequenceLogo";
          break;
        case "oninit":
          break;
        case "pdbseq":
          appName = "PDBSEQ";
          break;
        case "relaxedidmatch":
          break;
        case "resolvetocodebase":
          break;
        case "rgb":
          appName = "RGB";
          break;
        case "scaleproteinascdna":
          appName = "scaleProteinAsCdna";
          break;
        case "separator":
          break;
        case "sequence":
          break;
        case "sortby":
          break;
        case "sortbytree":
          value = checkTF(value);
          appletName = null; // taken care of by Jalview
          break;
        case "format":
          break;
        case "scorefile":
          appName = "scoreFile";
          value = resourcePath + value;
          break;
        case "uppercase":
          appName = "upperCase";
          break;
        case "userdefinedcolour":
          appName = "colour";
          break;
        case "widthscale":
          appName = "widthScale";
          break;
        case "windowheight":
          appName = "windowHeight";
          break;
        case "windowwidth":
          appName = "windowWidth";
          break;
        case "wrap":
          break;

        // cases not handled in doc/parameters.xlsx
        case "debug":
          value = checkTF(value);
          break;
        case "file":
          appName = "open";
          value = resourcePath + value;
          break;
        case "file2":
          value = resourcePath + value;
          break;
        case "embedded":
          value = checkTF(value);
          break;
        case "showannotation":
          appName = "SHOW_ANNOTATIONS";
          value = checkTF(value);
          break;
        case "showbutton":
          value = checkTF(value);
          break;
        case "showconsensus":
          appName = "SHOW_CONSENSUS_LOGO";
          value = checkTF(value);
          break;
        case "showconsensushistogram":
          appName = "SHOW_CONSENSUS_HISTOGRAM";
          value = checkTF(value);
          break;
        case "showconservation":
          appName = "SHOW_CONSERVATION";
          value = checkTF(value);
          break;
        case "showfeaturegroups":
          value = checkTF(value);
          break;
        case "showfeaturesettings":
          appName = "showFeatureSettings";
          value = checkTF(value);
          break;
        case "showfullid":
          appName = "showFullId";
          value = checkTF(value);
          break;
        case "showgroupconsensus":
          appName = "SHOW_GROUP_CONSENSUS";
          value = checkTF(value);
          break;
        case "showgroupconservation":
          appName = "SHOW_GROUP_CONSERVATION";
          value = checkTF(value);
          break;
        case "showoccupancy":
          appName = "SHOW_OCCUPANCY";
          value = checkTF(value);
          break;
        case "showquality":
          appName = "SHOW_QUALITY";
          value = checkTF(value);
          break;
        case "showsequencelogo":
          appName = "showSequenceLogo";
          value = checkTF(value);
          break;
        case "showtreebootstraps":
          appName = "showTreeBootstraps";
          value = checkTF(value);
          break;
        case "showtreedistances":
          appName = "showTreeDistances";
          value = checkTF(value);
          break;
        case "showunconserved":
          appName = "showUnconserved";
          value = checkTF(value);
          break;
        case "showunlinkedtreenodes":
          appName = "showUnlinkedTreeNodes";
          value = checkTF(value);
          break;
        default:
          break;
        }
        if (value != null)
        {
          vargs.add(appName);
          if (value != "true")
          {
            vargs.add(value);
          }
        }
        if (value == null)
        {
          value = "false";
        }
        System.out.println("AppletParams appName=" + appName + "appletName="
                + appletName + " value=" + value);
        Cache.setPropertyNoSave(appName, value);
        if (appletName != null)
        {
          appletParams.put(appletName, value);
        }
      }
    }
    return appletParams;
  }

  /**
   * Check for a single-argument option.
   * 
   * @param value
   * @return "true" or null
   */
  private static String checkTF(String value)
  {
    return (value.toLowerCase() == "true" ? "true" : null);
  }

  /**
   * Crude applet innerHTML parser
   * 
   * @param tag
   * @param attr
   * @return
   */
  private static String getAttr(String tag, String attr)
  {
    int pt = tag.indexOf(attr + "=\"");
    if (pt < 0)
    {
      System.out
              .println("AppletParams did not read " + attr + " in " + tag);
      return null;
    }
    // <param name="sortByTree" value="True"/>
    int pt1 = pt + attr.length() + 2;
    int pt2 = tag.indexOf("\"", pt1);
    return (pt < 0 ? null : tag.substring(pt1, pt2));
  }

  public static void main(String[] args)
  {
    new AppletParams("<applet\r\n"
            + "    code=\"jalview.bin.JalviewLite\" width=\"140\" height=\"35\"\r\n"
            + "    archive=\"jalviewApplet.jar,JmolApplet-14.6.4_2016.10.26.jar,java-json.jar,json_simple-1.1.jar\">  \r\n"
            + "  <param name=\"permissions\" value=\"sandbox\"/>\r\n"
            + "  <param name=\"file\" value=\"uniref50.fa\"/>\r\n"
            + "  <param name=\"treeFile\" value=\"ferredoxin.nw\"/>\r\n"
            + "  <param name=\"userDefinedColour\" value=\"C=yellow; R,K,H=FF5555; D,E=5555FF\"/>\r\n"
            + "  <param name=\"sortByTree\" value=\"True\"/>\r\n"
            + "  <param name=\"showSequenceLogo\" value=\"true\"/>\r\n"
            + "  <param name=\"showGroupConsensus\" value=\"true\"/>\r\n"
            + "  <param name=\"showFullId\" value=\"false\"/>\r\n"
            + "    <param name=\"linkLabel_1\" value=\"Uniprot\"/>\r\n"
            + "    <param name=\"linkUrl_1\" value=\"http://www.uniprot.org/uniprot/$SEQUENCE_ID$\"/>\r\n"
            + "    <param name=\"linkLabel_2\" value=\"EMBL-EBI Search\"/>\r\n"
            + "    <param name=\"linkUrl_2\" value=\"http://www.ebi.ac.uk/ebisearch/search.ebi?db=allebi&query=$SEQUENCE_ID$\"/>\r\n"
            + "    <param name=\"APPLICATION_URL\" value=\"http://www.jalview.org/services/launchApp\"/>\r\n"
            + "     </applet>");
  }

}